package com.dj.service.sys.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.dj.comparator.AreaComparator;
//import com.dj.comparator.DeptComparator;
import com.dj.dao.base.BaseDaoI;
import com.dj.model.sys.Tarea;
import com.dj.pageModel.base.TreeNode;
import com.dj.pageModel.sys.Area;
import com.dj.service.base.impl.BaseServiceImpl;
import com.dj.service.sys.AreaServiceI;


@Service("areaService")
public class AreaServiceImpl extends BaseServiceImpl implements AreaServiceI {

	private BaseDaoI<Tarea> areaDao;

	public BaseDaoI<Tarea> getAreaDao() {
		return areaDao;
	}

	@Autowired
	public void setAreaDao(BaseDaoI<Tarea> areaDao) {
		this.areaDao = areaDao;
	}

	@Transactional(readOnly = true, propagation = Propagation.SUPPORTS)
	public List<Area> findAll() {
		List<Area> ld = new ArrayList<Area>();
		String hql = "from Tarea t order by t.cseq";
		List<Tarea> ltd = areaDao.find(hql);
		for (Tarea td : ltd) {
			Area d = new Area();
			d.setCid(td.getCid());
			d.setCname(td.getCname());
			d.setCiconcls(td.getCiconcls());
			if(td.getTarea()!=null){
				d.setPid(td.getTarea().getCid());
			}
			ld.add(d);
		}
		return ld;
	}

	@Transactional(readOnly = true, propagation = Propagation.SUPPORTS)
	public List<TreeNode> tree(Area area, Boolean b) {
		List<Object> param = new ArrayList<Object>();
		String hql = "from Tarea t where t.tarea is null order by t.cseq";
		if (area != null && area.getId() != null && !area.getId().trim().equals("")) {
			hql = "from Tarea t where t.tarea.cid = ? order by t.cseq";
			param.add(area.getId());
		}
		List<Tarea> l = areaDao.find(hql, param);
		List<TreeNode> tree = new ArrayList<TreeNode>();
		for (Tarea t : l) {
			tree.add(this.tree(t, b));
		}
		return tree;
	}

	private TreeNode tree(Tarea t, boolean recursive) {
		TreeNode node = new TreeNode();
		node.setId(t.getCid());
		node.setText(t.getCname());
		if (t.getCiconcls() != null) {
			node.setIconCls(t.getCiconcls());
		} else {
			node.setIconCls("");
		}
		if (t.getTareas() != null && t.getTareas().size() > 0) {
			node.setState("closed");
			if (recursive) {// 递归查询子节点
				List<Tarea> l = new ArrayList<Tarea>(t.getTareas());
				Collections.sort(l, new AreaComparator());// 排序
				List<TreeNode> children = new ArrayList<TreeNode>();
				for (Tarea r : l) {
					TreeNode tn = tree(r, true);
					children.add(tn);
				}
				node.setChildren(children);
				node.setState("open");
			}
		}
		return node;
	}

	@Transactional(readOnly = true, propagation = Propagation.SUPPORTS)
	public List<Area> treegrid(Area area) {
		List<Tarea> l;
		if (area != null && area.getId() != null) {
			l = areaDao.find("from Tarea t where t.tarea.cid = ? order by t.cseq", new Object[] { area.getId() });
		} else {
			l = areaDao.find("from Tarea t where t.tarea is null order by t.cseq");
		}
		return changeModel(l);
	}

	private List<Area> changeModel(List<Tarea> Tareas) {
		List<Area> l = new ArrayList<Area>();
		if (Tareas != null && Tareas.size() > 0) {
			for (Tarea t : Tareas) {
				Area m = new Area();
				BeanUtils.copyProperties(t, m);
				if (t.getTarea() != null) {
					m.setPid(t.getTarea().getCid());
					m.setPname(t.getTarea().getCname());
				}
				if (countChildren(t.getCid()) > 0) {
					m.setState("closed");
				}
				if (t.getCiconcls() == null) {
					m.setIconCls("");
				} else {
					m.setIconCls(t.getCiconcls());
				}
				l.add(m);
			}
		}
		return l;
	}

	/**
	 * 统计有多少子节点
	 */
	private Long countChildren(String pid) {
		return areaDao.count("select count(*) from Tarea t where t.tarea.cid = ?", new Object[] { pid });
	}

	public void delete(Area area) {
		del(area.getCid());
	}

	private void del(String cid) {
		Tarea t = areaDao.get(Tarea.class, cid);
		if (t != null) {
			Set<Tarea> areas = t.getTareas();
			if (areas != null && !areas.isEmpty()) {
				// 说明当前菜单下面还有子菜单
				for (Tarea tarea : areas) {
					del(tarea.getCid());
				}
			}
			areaDao.delete(t);
		}
	}

	public void add(Area area) {
		Tarea t = new Tarea();
		BeanUtils.copyProperties(area, t);
		t.setCid(UUID.randomUUID().toString());
		if (area.getPid() != null) {
			t.setTarea(areaDao.get(Tarea.class, area.getPid()));
		}
		if (area.getIconCls() != null) {
			t.setCiconcls(area.getIconCls());
		}
		areaDao.save(t);
	}

	public void edit(Area area) {
		Tarea t = areaDao.get(Tarea.class, area.getCid());
		BeanUtils.copyProperties(area, t);
		if (area.getIconCls() != null) {
			t.setCiconcls(area.getIconCls());
		}
		if (area.getPid() != null && !area.getPid().equals(area.getCid())) {
			Tarea pt = areaDao.get(Tarea.class, area.getPid());
			if (pt != null) {
				if (isDown(t, pt)) {// 要将当前节点修改到当前节点的子节点中
					Set<Tarea> tareas = t.getTareas();// 当前要修改的节点的所有下级节点
					if (tareas != null && tareas.size() > 0) {
						for (Tarea tarea : tareas) {
							if (tarea != null) {
								tarea.setTarea(null);
							}
						}
					}
				}
				t.setTarea(pt);
			}

		}
	}

	/**
	 * 判断是否是将当前节点修改到当前节点的子节点
	 * 
	 * @param t
	 * @param pt
	 * @return
	 */
	private boolean isDown(Tarea t, Tarea pt) {
		if (pt.getTarea() != null) {
			if (pt.getTarea().getCid().equals(t.getCid())) {
				return true;
			} else {
				return isDown(t, pt.getTarea());
			}
		}
		return false;
	}

}
